@@ -1,16 +1,51 @@
 /* Some misc util functions for audio DSP work, written by Steve Harris,
-   December 2000
-
-   steve@plugin.org.uk
-*/
+ * December 2000
+ *
+ *  steve@plugin.org.uk
+ */
 
 #ifndef LADSPA_UTIL_H
 #define LADSPA_UTIL_H
 
 #include <math.h>
+#include <stdint.h>
 
 #include "config.h"
 
+// 16.16 fixpoint
+typedef union {
+	int32_t all;
+	struct {
+#ifdef WORDS_BIGENDIAN
+		int16_t in;
+		uint16_t fr;
+#else
+		uint16_t fr;
+		int16_t in;
+#endif
+	} part;
+} fixp16;
+
+// 32.32 fixpoint
+typedef union {
+	int64_t all;
+	struct {
+#ifdef WORDS_BIGENDIAN
+		int32_t in;
+		uint32_t fr;
+#else
+		uint32_t fr;
+		int32_t in;
+#endif
+	} part;
+} fixp32;
+
+/* 32 bit "pointer cast" union */
+typedef union {
+        float f;
+        int32_t i;
+} ls_pcast32;
+
 // Sometimes it doesn't get defined, even though it eists and C99 is declared
 long int lrintf (float x);
 
@@ -18,7 +53,18 @@ long int lrintf (float x);
 #define LN2R 1.442695041f
 
 // Denormalise floats, only actually needed for PIII and recent PowerPC
-#define FLUSH_TO_ZERO(fv) (((*(unsigned int*)&(fv))&0x7f800000)==0)?0.0f:(fv)
+//#define FLUSH_TO_ZERO(fv) (((*(unsigned int*)&(fv))&0x7f800000)==0)?0.0f:(fv)
+
+static inline float flush_to_zero(float f)
+{
+	ls_pcast32 v;
+
+	v.f = f;
+
+	// original: return (v.i & 0x7f800000) == 0 ? 0.0f : f;
+	// version from Tim Blechmann
+	return (v.i & 0x7f800000) < 0x08000000 ? 0.0f : f;
+}
 
 /* A set of branchless clipping operations from Laurent de Soras */
 
@@ -85,29 +131,22 @@ static inline float cube_interp(const float fr, const float inm1, const float
  * music-dsp list */
 static inline float f_sin_sq(float angle)
 {
-        const float asqr = angle * angle;
-        float result = -2.39e-08f;
+	const float asqr = angle * angle;
+	float result = -2.39e-08f;
 
-        result *= asqr;
-        result += 2.7526e-06f;
-        result *= asqr;
-        result -= 1.98409e-04f;
-        result *= asqr;
-        result += 8.3333315e-03f;
-        result *= asqr;
-        result -= 1.666666664e-01f;
-        result *= asqr;
-        result += 1.0f;
-        result *= angle;
+	result *= asqr;
+	result += 2.7526e-06f;
+	result *= asqr;
+	result -= 1.98409e-04f;
+	result *= asqr;
+	result += 8.3333315e-03f;
+	result *= asqr;
+	result -= 1.666666664e-01f;
+	result *= asqr;
+	result += 1.0f;
+	result *= angle;
 
-        return result * result;
-}
-
-// Truncate float to int
-static inline int f_trunc(float f) {
-        f -= 0.5f;
-        f += (3<<22);
-        return *((int*)&f) - 0x4b400000;
+	return result * result;
 }
 
 #ifdef HAVE_LRINTF
@@ -117,33 +156,29 @@ static inline int f_trunc(float f) {
 #else
 
 // Round float to int using IEEE int* hack
-static inline int f_round(float f) {
-        f += (3<<22);
-        return *((int*)&f) - 0x4b400000;
+static inline int f_round(float f)
+{
+	ls_pcast p;
+
+	p.f = f;
+	p.f += (3<<22);
+
+	return p.i - 0x4b400000;
 }
 
 #endif
 
-// 16.16 fixpoint
-typedef union {
-	int all;
-	struct {
-		unsigned short fr;
-		short in;
-	} part;
-} fixp16;
-
-// 32.32 fixpoint
-typedef union {
-	long long all;
-	struct {
-		unsigned int fr;
-		int in;
-	} part;
-} fixp32;
-
+// Truncate float to int
+static inline int f_trunc(float f)
+{
+	return f_round(floorf(f));
+}
 
 /* Andrew Simper's pow(2, x) aproximation from the music-dsp list */
+
+#if 0
+
+/* original */
 static inline float f_pow2(float x)
 {
 	long *px = (long*)(&x); // store address of float as long pointer
@@ -159,6 +194,29 @@ static inline float f_pow2(float x)
 	return x;
 }
 
+#else
+
+/* union version */
+static inline float f_pow2(float x)
+{
+	ls_pcast32 *px, tx, lx;
+	float dx;
+
+	px = (ls_pcast32 *)&x; // store address of float as long pointer
+	tx.f = (x-0.5f) + (3<<22); // temporary value for truncation
+	lx.i = tx.i - 0x4b400000; // integer power of 2
+	dx = x - (float)lx.i; // float remainder of power of 2
+
+	x = 1.0f + dx * (0.6960656421638072f + // cubic apporoximation of 2^x
+		   dx * (0.224494337302845f +  // for x in the range [0, 1]
+		   dx * (0.07944023841053369f)));
+	(*px).i += (lx.i << 23); // add integer power of 2 to exponent
+
+	return (*px).f;
+}
+
+#endif
+
 /* Fast exponentiation function, y = e^x */
 #define f_exp(x) f_pow2(x * LN2R)
 
